@@ -26,8 +26,6 @@
 #include <rocksdb/transaction_log.h>
 #include <rocksdb/write_batch.h>
 
-#include <chrono>
-#include <map>
 #include <memory>
 #include <string>
 #include <thread>
@@ -35,17 +33,14 @@
 #include <vector>
 
 #include "batch_sender.h"
-#include "encoding.h"
-#include "parse_util.h"
 #include "server/server.h"
-#include "slot_import.h"
 #include "status.h"
 #include "storage/redis_db.h"
 #include "unique_fd.h"
 
 enum class MigrationType {
   /// Use Redis commands to migrate data.
-  /// It will trying to extract commands from existing data and log, then replay
+  /// It will try to extract commands from existing data and log, then replay
   /// them on the destination node.
   kRedisCommand = 0,
   /// Using raw key-value and "APPLYBATCH" command in kvrocks to migrate data.
