@@ -65,7 +65,7 @@ class CommandCluster : public Commander {
     return {Status::RedisParseErr, "CLUSTER command, CLUSTER INFO|NODES|SLOTS|KEYSLOT|RESET|REPLICAS"};
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     if (!srv->GetConfig()->cluster_enabled) {
       return {Status::RedisExecErr, "Cluster mode is not enabled"};
     }
@@ -116,14 +116,14 @@ class CommandCluster : public Commander {
       // TODO: support multiple slot ranges
       Status s = srv->cluster->ImportSlotRange(conn, slot_ranges_[0], state_);
       if (s.IsOK()) {
-        *output = redis::SimpleString("OK");
+        *output = redis::RESP_OK;
       } else {
         return s;
       }
     } else if (subcommand_ == "reset") {
       Status s = srv->cluster->Reset();
       if (s.IsOK()) {
-        *output = redis::SimpleString("OK");
+        *output = redis::RESP_OK;
       } else {
         return s;
       }
@@ -244,7 +244,7 @@ class CommandClusterX : public Commander {
     return {Status::RedisParseErr, "CLUSTERX command, CLUSTERX VERSION|MYID|SETNODEID|SETNODES|SETSLOT|MIGRATE"};
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     if (!srv->GetConfig()->cluster_enabled) {
       return {Status::RedisExecErr, "Cluster mode is not enabled"};
     }
@@ -258,7 +258,7 @@ class CommandClusterX : public Commander {
       Status s = srv->cluster->SetClusterNodes(nodes_str_, set_version_, force_);
       if (s.IsOK()) {
         need_persist_nodes_info = true;
-        *output = redis::SimpleString("OK");
+        *output = redis::RESP_OK;
       } else {
         return s;
       }
@@ -266,7 +266,7 @@ class CommandClusterX : public Commander {
       Status s = srv->cluster->SetNodeId(args_[2]);
       if (s.IsOK()) {
         need_persist_nodes_info = true;
-        *output = redis::SimpleString("OK");
+        *output = redis::RESP_OK;
       } else {
         return s;
       }
@@ -274,7 +274,7 @@ class CommandClusterX : public Commander {
       Status s = srv->cluster->SetSlotRanges(slot_ranges_, args_[4], set_version_);
       if (s.IsOK()) {
         need_persist_nodes_info = true;
-        *output = redis::SimpleString("OK");
+        *output = redis::RESP_OK;
       } else {
         return s;
       }
@@ -293,7 +293,7 @@ class CommandClusterX : public Commander {
         if (sync_migrate_) {
           return {Status::BlockingCmd};
         }
-        *output = redis::SimpleString("OK");
+        *output = redis::RESP_OK;
       } else {
         return s;
       }
@@ -329,8 +329,9 @@ static uint64_t GenerateClusterFlag(uint64_t flags, const std::vector<std::strin
 
 class CommandReadOnly : public Commander {
  public:
-  Status Execute([[maybe_unused]] Server *srv, Connection *conn, std::string *output) override {
-    *output = redis::SimpleString("OK");
+  Status Execute([[maybe_unused]] engine::Context &ctx, [[maybe_unused]] Server *srv, Connection *conn,
+                 std::string *output) override {
+    *output = redis::RESP_OK;
     conn->EnableFlag(redis::Connection::kReadOnly);
     return Status::OK();
   }
@@ -338,8 +339,9 @@ class CommandReadOnly : public Commander {
 
 class CommandReadWrite : public Commander {
  public:
-  Status Execute([[maybe_unused]] Server *srv, Connection *conn, std::string *output) override {
-    *output = redis::SimpleString("OK");
+  Status Execute([[maybe_unused]] engine::Context &ctx, [[maybe_unused]] Server *srv, Connection *conn,
+                 std::string *output) override {
+    *output = redis::RESP_OK;
     conn->DisableFlag(redis::Connection::kReadOnly);
     return Status::OK();
   }
@@ -347,18 +349,18 @@ class CommandReadWrite : public Commander {
 
 class CommandAsking : public Commander {
  public:
-  Status Execute([[maybe_unused]] Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, [[maybe_unused]] Server *srv, Connection *conn,
+                 std::string *output) override {
     conn->EnableFlag(redis::Connection::kAsking);
-    *output = redis::SimpleString("OK");
+    *output = redis::RESP_OK;
     return Status::OK();
   }
 };
 
-REDIS_REGISTER_COMMANDS(Cluster,
-                        MakeCmdAttr<CommandCluster>("cluster", -2, "cluster no-script", 0, 0, 0, GenerateClusterFlag),
-                        MakeCmdAttr<CommandClusterX>("clusterx", -2, "cluster no-script", 0, 0, 0, GenerateClusterFlag),
-                        MakeCmdAttr<CommandReadOnly>("readonly", 1, "cluster no-multi", 0, 0, 0),
-                        MakeCmdAttr<CommandReadWrite>("readwrite", 1, "cluster no-multi", 0, 0, 0),
-                        MakeCmdAttr<CommandAsking>("asking", 1, "cluster", 0, 0, 0), )
+REDIS_REGISTER_COMMANDS(Cluster, MakeCmdAttr<CommandCluster>("cluster", -2, "no-script", NO_KEY, GenerateClusterFlag),
+                        MakeCmdAttr<CommandClusterX>("clusterx", -2, "no-script", NO_KEY, GenerateClusterFlag),
+                        MakeCmdAttr<CommandReadOnly>("readonly", 1, "no-multi", NO_KEY),
+                        MakeCmdAttr<CommandReadWrite>("readwrite", 1, "no-multi", NO_KEY),
+                        MakeCmdAttr<CommandAsking>("asking", 1, "", NO_KEY), )
 
 }  // namespace redis
