@@ -34,6 +34,7 @@
 #include "search/sql_transformer.h"
 #include "server/redis_reply.h"
 #include "server/server.h"
+#include "status.h"
 #include "string_util.h"
 #include "tao/pegtl/string_input.hpp"
 
@@ -179,12 +180,12 @@ class CommandFTCreate : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     index_info_->ns = conn->GetNamespace();
-    engine::Context ctx(srv->storage);
+
     GET_OR_RET(srv->index_mgr.Create(ctx, std::move(index_info_)));
 
-    output->append(redis::SimpleString("OK"));
+    output->append(redis::RESP_OK);
     return Status::OK();
   };
 
@@ -275,7 +276,7 @@ class CommandFTExplainSQL : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     auto plan = GET_OR_RET(srv->index_mgr.GeneratePlan(std::move(ir_), conn->GetNamespace()));
 
     if (format_ == SIMPLE) {
@@ -306,7 +307,7 @@ class CommandFTSearchSQL : public Commander {
 
     return Status::OK();
   }
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     auto results = GET_OR_RET(srv->index_mgr.Search(std::move(ir_), conn->GetNamespace()));
 
     DumpQueryResult(results, output);
@@ -332,7 +333,7 @@ static StatusOr<std::unique_ptr<kqir::Node>> ParseRediSearchQuery(const std::vec
 
   kqir::ParamMap param_map;
   while (parser.Good()) {
-    if (parser.EatEqICase("RETURNS")) {
+    if (parser.EatEqICase("RETURN")) {
       auto count = GET_OR_RET(parser.TakeInt<size_t>());
 
       for (size_t i = 0; i < count; ++i) {
@@ -384,7 +385,7 @@ class CommandFTExplain : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     CHECK(ir_);
     auto plan = GET_OR_RET(srv->index_mgr.GeneratePlan(std::move(ir_), conn->GetNamespace()));
 
@@ -403,7 +404,7 @@ class CommandFTSearch : public Commander {
     return Status::OK();
   }
 
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     CHECK(ir_);
     auto results = GET_OR_RET(srv->index_mgr.Search(std::move(ir_), conn->GetNamespace()));
 
@@ -417,7 +418,7 @@ class CommandFTSearch : public Commander {
 };
 
 class CommandFTInfo : public Commander {
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     const auto &index_map = srv->index_mgr.index_map;
     const auto &index_name = args_[1];
 
@@ -427,24 +428,55 @@ class CommandFTInfo : public Commander {
     }
 
     const auto &info = iter->second;
-    output->append(MultiLen(8));
+    output->append(MultiLen(6));
 
     output->append(redis::SimpleString("index_name"));
     output->append(redis::BulkString(info->name));
 
-    output->append(redis::SimpleString("on_data_type"));
+    output->append(redis::SimpleString("index_definition"));
+    output->append(redis::MultiLen(4));
+    output->append(redis::SimpleString("key_type"));
     output->append(redis::BulkString(RedisTypeNames[(size_t)info->metadata.on_data_type]));
-
     output->append(redis::SimpleString("prefixes"));
     output->append(redis::ArrayOfBulkStrings(info->prefixes.prefixes));
 
     output->append(redis::SimpleString("fields"));
     output->append(MultiLen(info->fields.size()));
     for (const auto &[_, field] : info->fields) {
-      output->append(MultiLen(2));
+      output->append(MultiLen(6));
+      output->append(redis::SimpleString("identifier"));
       output->append(redis::BulkString(field.name));
+      output->append(redis::SimpleString("type"));
       auto type = field.metadata->Type();
       output->append(redis::BulkString(std::string(type.begin(), type.end())));
+      output->append(redis::SimpleString("properties"));
+      if (auto tag = field.MetadataAs<TagFieldMetadata>()) {
+        output->append(redis::MultiLen(4));
+        output->append(redis::SimpleString("separator"));
+        output->append(redis::BulkString(std::string(1, tag->separator)));
+        output->append(redis::SimpleString("case_sensitive"));
+        output->append(conn->Bool(tag->case_sensitive));
+      } else if (auto vec = field.MetadataAs<HnswVectorFieldMetadata>()) {
+        output->append(redis::MultiLen(16));
+        output->append(redis::SimpleString("algorithm"));
+        output->append(redis::SimpleString("HNSW"));
+        output->append(redis::SimpleString("vector_type"));
+        output->append(redis::SimpleString(VectorTypeToString(vec->vector_type)));
+        output->append(redis::SimpleString("dim"));
+        output->append(redis::Integer(vec->dim));
+        output->append(redis::SimpleString("distance_metric"));
+        output->append(redis::SimpleString(DistanceMetricToString(vec->distance_metric)));
+        output->append(redis::SimpleString("m"));
+        output->append(redis::Integer(vec->m));
+        output->append(redis::SimpleString("ef_construction"));
+        output->append(redis::Integer(vec->ef_construction));
+        output->append(redis::SimpleString("ef_runtime"));
+        output->append(redis::Integer(vec->ef_runtime));
+        output->append(redis::SimpleString("epsilon"));
+        output->append(conn->Double(vec->epsilon));
+      } else {
+        output->append(redis::MultiLen(0));
+      }
     }
 
     return Status::OK();
@@ -452,7 +484,7 @@ class CommandFTInfo : public Commander {
 };
 
 class CommandFTList : public Commander {
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute([[maybe_unused]] engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     const auto &index_map = srv->index_mgr.index_map;
 
     std::vector<std::string> results;
@@ -469,25 +501,41 @@ class CommandFTList : public Commander {
 };
 
 class CommandFTDrop : public Commander {
-  Status Execute(Server *srv, Connection *conn, std::string *output) override {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
     const auto &index_name = args_[1];
 
-    GET_OR_RET(srv->index_mgr.Drop(index_name, conn->GetNamespace()));
+    GET_OR_RET(srv->index_mgr.Drop(ctx, index_name, conn->GetNamespace()));
 
-    output->append(SimpleString("OK"));
+    output->append(redis::RESP_OK);
 
     return Status::OK();
   };
 };
 
-// REDIS_REGISTER_COMMANDS(Search,
-//                        MakeCmdAttr<CommandFTCreate>("ft.create", -2, "write exclusive no-multi no-script", 0, 0, 0),
-//                        MakeCmdAttr<CommandFTSearchSQL>("ft.searchsql", -2, "read-only", 0, 0, 0),
-//                        MakeCmdAttr<CommandFTSearch>("ft.search", -3, "read-only", 0, 0, 0),
-//                        MakeCmdAttr<CommandFTExplainSQL>("ft.explainsql", -2, "read-only", 0, 0, 0),
-//                        MakeCmdAttr<CommandFTExplain>("ft.explain", -3, "read-only", 0, 0, 0),
-//                        MakeCmdAttr<CommandFTInfo>("ft.info", 2, "read-only", 0, 0, 0),
-//                        MakeCmdAttr<CommandFTList>("ft._list", 1, "read-only", 0, 0, 0),
-//                        MakeCmdAttr<CommandFTDrop>("ft.dropindex", 2, "write exclusive no-multi no-script", 0, 0, 0));
+class CommandFTTagVals : public Commander {
+  Status Execute(engine::Context &ctx, Server *srv, Connection *conn, std::string *output) override {
+    const auto &index_name = args_[1];
+    const auto &tag_field_name = args_[2];
+    auto field_values = GET_OR_RET(srv->index_mgr.TagValues(ctx, index_name, tag_field_name, conn->GetNamespace()));
+
+    std::vector<std::string> result_vec(field_values.begin(), field_values.end());
+
+    *output = conn->SetOfBulkStrings(result_vec);
+
+    return Status::OK();
+  };
+};
+
+REDIS_REGISTER_COMMANDS(Search,
+                        MakeCmdAttr<CommandFTCreate>("ft.create", -2, "write exclusive no-multi no-script slow",
+                                                     NO_KEY),
+                        MakeCmdAttr<CommandFTSearchSQL>("ft.searchsql", -2, "read-only", NO_KEY),
+                        MakeCmdAttr<CommandFTSearch>("ft.search", -3, "read-only", NO_KEY),
+                        MakeCmdAttr<CommandFTExplainSQL>("ft.explainsql", -2, "read-only", NO_KEY),
+                        MakeCmdAttr<CommandFTExplain>("ft.explain", -3, "read-only", NO_KEY),
+                        MakeCmdAttr<CommandFTInfo>("ft.info", 2, "read-only", NO_KEY),
+                        MakeCmdAttr<CommandFTList>("ft._list", 1, "read-only", NO_KEY),
+                        MakeCmdAttr<CommandFTDrop>("ft.dropindex", 2, "write exclusive no-multi no-script", NO_KEY),
+                        MakeCmdAttr<CommandFTTagVals>("ft.tagvals", 3, "read-only slow", NO_KEY));
 
 }  // namespace redis
