@@ -46,26 +46,43 @@
 
 class Server;
 
+namespace engine {
+struct Context;
+}
+
 namespace redis {
 
 class Connection;
 struct CommandAttributes;
 
 enum CommandFlags : uint64_t {
-  kCmdWrite = 1ULL << 0,           // "write" flag
-  kCmdReadOnly = 1ULL << 1,        // "read-only" flag
-  kCmdReplication = 1ULL << 2,     // "replication" flag
-  kCmdPubSub = 1ULL << 3,          // "pub-sub" flag
-  kCmdScript = 1ULL << 4,          // "script" flag
-  kCmdLoading = 1ULL << 5,         // "ok-loading" flag
-  kCmdMulti = 1ULL << 6,           // "multi" flag
-  kCmdExclusive = 1ULL << 7,       // "exclusive" flag
-  kCmdNoMulti = 1ULL << 8,         // "no-multi" flag
-  kCmdNoScript = 1ULL << 9,        // "no-script" flag
-  kCmdROScript = 1ULL << 10,       // "ro-script" flag for read-only script commands
-  kCmdCluster = 1ULL << 11,        // "cluster" flag
-  kCmdNoDBSizeCheck = 1ULL << 12,  // "no-dbsize-check" flag
-  kCmdSlow = 1ULL << 13,           // "slow" flag
+  // "write" flag, for any command that performs rocksdb writing ops
+  kCmdWrite = 1ULL << 0,
+  // "read-only" flag, for any command that performs rocksdb reading ops
+  // and doesn't perform rocksdb writing ops
+  kCmdReadOnly = 1ULL << 1,
+  // "ok-loading" flag, for any command that can be executed while
+  // the db is in loading phase
+  kCmdLoading = 1ULL << 5,
+  // "bypass-multi" flag, for commands that can be executed in a MULTI scope,
+  // but these commands will NOT be queued and will be executed immediately
+  kCmdBypassMulti = 1ULL << 6,
+  // "exclusive" flag, for commands that should be executed execlusive globally
+  kCmdExclusive = 1ULL << 7,
+  // "no-multi" flag, for commands that cannot be executed in MULTI scope
+  kCmdNoMulti = 1ULL << 8,
+  // "no-script" flag, for commands that cannot be executed in scripting
+  kCmdNoScript = 1ULL << 9,
+  // "no-dbsize-check" flag, for commands that can ignore the db size checking
+  kCmdNoDBSizeCheck = 1ULL << 12,
+  // "slow" flag, for commands that run slowly,
+  // usually with a non-constant number of rocksdb ops
+  kCmdSlow = 1ULL << 13,
+  // "blocking" flag, for commands that don't perform db ops immediately,
+  // but block and wait for some event to happen before performing db ops
+  kCmdBlocking = 1ULL << 14,
+  // "auth" flag, for commands used for authentication
+  kCmdAuth = 1ULL << 15,
 };
 
 enum class CommandCategory : uint8_t {
@@ -100,8 +117,8 @@ class Commander {
   void SetArgs(const std::vector<std::string> &args) { args_ = args; }
   virtual Status Parse() { return Parse(args_); }
   virtual Status Parse([[maybe_unused]] const std::vector<std::string> &args) { return Status::OK(); }
-  virtual Status Execute([[maybe_unused]] Server *srv, [[maybe_unused]] Connection *conn,
-                         [[maybe_unused]] std::string *output) {
+  virtual Status Execute([[maybe_unused]] engine::Context &ctx, [[maybe_unused]] Server *srv,
+                         [[maybe_unused]] Connection *conn, [[maybe_unused]] std::string *output) {
     return {Status::RedisExecErr, errNotImplemented};
   }
 
@@ -137,9 +154,18 @@ struct CommandKeyRange {
   template <typename F>
   void ForEachKey(F &&f, const std::vector<std::string> &args) const {
     for (size_t i = first_key; last_key > 0 ? i <= size_t(last_key) : i <= args.size() + last_key; i += key_step) {
+      if (i >= args.size()) continue;
       std::forward<F>(f)(args[i]);
     }
   }
+
+  template <typename F>
+  void ForEachKeyIndex(F &&f, size_t arg_size) const {
+    for (size_t i = first_key; last_key > 0 ? i <= size_t(last_key) : i <= arg_size + last_key; i += key_step) {
+      if (i >= arg_size) continue;
+      std::forward<F>(f)(i);
+    }
+  }
 };
 
 using CommandKeyRangeGen = std::function<CommandKeyRange(const std::vector<std::string> &)>;
@@ -148,7 +174,58 @@ using CommandKeyRangeVecGen = std::function<std::vector<CommandKeyRange>(const s
 
 using AdditionalFlagGen = std::function<uint64_t(uint64_t, const std::vector<std::string> &)>;
 
+struct NoKeyInThisCommand {};
+static constexpr const NoKeyInThisCommand NO_KEY{};
+
 struct CommandAttributes {
+  CommandAttributes(std::string name, int arity, CommandCategory category, uint64_t flags, AdditionalFlagGen flag_gen,
+                    NoKeyInThisCommand, CommanderFactory factory)
+      : name(std::move(name)),
+        arity(arity),
+        category(category),
+        factory(std::move(factory)),
+        flags_(flags),
+        flag_gen_(std::move(flag_gen)),
+        key_range_{0, 0, 0} {}
+
+  CommandAttributes(std::string name, int arity, CommandCategory category, uint64_t flags, AdditionalFlagGen flag_gen,
+                    CommandKeyRange key_range, CommanderFactory factory)
+      : name(std::move(name)),
+        arity(arity),
+        category(category),
+        factory(std::move(factory)),
+        flags_(flags),
+        flag_gen_(std::move(flag_gen)),
+        key_range_(key_range) {
+    if (key_range.first_key <= 0 || key_range.key_step <= 0 ||
+        (key_range.last_key >= 0 && key_range.last_key < key_range.first_key)) {
+      std::cout << fmt::format("Encountered invalid key range in command {}", this->name) << std::endl;
+      std::abort();
+    }
+  }
+
+  CommandAttributes(std::string name, int arity, CommandCategory category, uint64_t flags, AdditionalFlagGen flag_gen,
+                    CommandKeyRangeGen key_range, CommanderFactory factory)
+      : name(std::move(name)),
+        arity(arity),
+        category(category),
+        factory(std::move(factory)),
+        flags_(flags),
+        flag_gen_(std::move(flag_gen)),
+        key_range_{-1, 0, 0},
+        key_range_gen_(std::move(key_range)) {}
+
+  CommandAttributes(std::string name, int arity, CommandCategory category, uint64_t flags, AdditionalFlagGen flag_gen,
+                    CommandKeyRangeVecGen key_range, CommanderFactory factory)
+      : name(std::move(name)),
+        arity(arity),
+        category(category),
+        factory(std::move(factory)),
+        flags_(flags),
+        flag_gen_(std::move(flag_gen)),
+        key_range_{-2, 0, 0},
+        key_range_vec_gen_(std::move(key_range)) {}
+
   // command name
   std::string name;
 
@@ -160,27 +237,14 @@ struct CommandAttributes {
   // category of this command, e.g. key, string, hash
   CommandCategory category;
 
-  // bitmap of enum CommandFlags
-  uint64_t flags;
-
-  // additional flags regarding to dynamic command arguments
-  AdditionalFlagGen flag_gen;
-
-  // static determined key range
-  CommandKeyRange key_range;
-
-  // if key_range.first_key == -1, key_range_gen is used instead
-  CommandKeyRangeGen key_range_gen;
-
-  // if key_range.first_key == -2, key_range_vec_gen is used instead
-  CommandKeyRangeVecGen key_range_vec_gen;
-
   // commander object generator
   CommanderFactory factory;
 
+  uint64_t InitialFlags() const { return flags_; }
+
   auto GenerateFlags(const std::vector<std::string> &args) const {
-    uint64_t res = flags;
-    if (flag_gen) res = flag_gen(res, args);
+    uint64_t res = flags_;
+    if (flag_gen_) res = flag_gen_(res, args);
     return res;
   }
 
@@ -188,26 +252,59 @@ struct CommandAttributes {
     return !((arity > 0 && cmd_size != arity) || (arity < 0 && cmd_size < -arity));
   }
 
-  template <typename F>
-  void ForEachKeyRange(F &&f, const std::vector<std::string> &args) const {
-    if (key_range.first_key > 0) {
-      std::forward<F>(f)(args, key_range);
-    } else if (key_range.first_key == -1) {
-      redis::CommandKeyRange range = key_range_gen(args);
+  StatusOr<CommandKeyRange> InitialKeyRange() const {
+    if (key_range_.first_key >= 0) return key_range_;
+    return {Status::NotOK, "key range is unavailable without command arguments"};
+  }
+
+  // the command arguments must be parsed and in valid syntax
+  // before this method is called, otherwise the behavior is UNDEFINED
+  template <typename F, typename G>
+  void ForEachKeyRange(F &&f, const std::vector<std::string> &args, G &&g) const {
+    if (key_range_.first_key > 0) {
+      std::forward<F>(f)(args, key_range_);
+    } else if (key_range_.first_key == -1) {
+      redis::CommandKeyRange range = key_range_gen_(args);
 
       if (range.first_key > 0) {
         std::forward<F>(f)(args, range);
       }
-    } else if (key_range.first_key == -2) {
-      std::vector<redis::CommandKeyRange> vec_range = key_range_vec_gen(args);
+    } else if (key_range_.first_key == -2) {
+      std::vector<redis::CommandKeyRange> vec_range = key_range_vec_gen_(args);
 
       for (const auto &range : vec_range) {
         if (range.first_key > 0) {
           std::forward<F>(f)(args, range);
         }
       }
+    } else if (key_range_.first_key == 0) {
+      // otherwise, if there's no key inside the command arguments
+      // e.g. FLUSHALL, with "write" flag but no key specified
+      std::forward<G>(g)(args);
     }
   }
+
+  template <typename F>
+  void ForEachKeyRange(F &&f, const std::vector<std::string> &args) const {
+    ForEachKeyRange(std::forward<F>(f), args, [](const auto &) {});
+  }
+
+ private:
+  // bitmap of enum CommandFlags
+  uint64_t flags_;
+
+  // additional flags regarding to dynamic command arguments
+  AdditionalFlagGen flag_gen_;
+
+  // static determined key range
+  // if key_range.first_key == 0, there's no key in this command args
+  CommandKeyRange key_range_;
+
+  // if key_range.first_key == -1, key_range_gen is used instead
+  CommandKeyRangeGen key_range_gen_;
+
+  // if key_range.first_key == -2, key_range_vec_gen is used instead
+  CommandKeyRangeVecGen key_range_vec_gen_;
 };
 
 using CommandMap = std::map<std::string, const CommandAttributes *>;
@@ -220,29 +317,29 @@ inline uint64_t ParseCommandFlags(const std::string &description, const std::str
       flags |= kCmdWrite;
     else if (flag == "read-only")
       flags |= kCmdReadOnly;
-    else if (flag == "replication")
-      flags |= kCmdReplication;
-    else if (flag == "pub-sub")
-      flags |= kCmdPubSub;
     else if (flag == "ok-loading")
       flags |= kCmdLoading;
     else if (flag == "exclusive")
       flags |= kCmdExclusive;
-    else if (flag == "multi")
-      flags |= kCmdMulti;
+    else if (flag == "bypass-multi")
+      flags |= kCmdBypassMulti;
     else if (flag == "no-multi")
       flags |= kCmdNoMulti;
     else if (flag == "no-script")
       flags |= kCmdNoScript;
-    else if (flag == "ro-script")
-      flags |= kCmdROScript;
-    else if (flag == "cluster")
-      flags |= kCmdCluster;
     else if (flag == "no-dbsize-check")
       flags |= kCmdNoDBSizeCheck;
     else if (flag == "slow")
       flags |= kCmdSlow;
-    else {
+    else if (flag == "auth")
+      flags |= kCmdAuth;
+    else if (flag == "blocking") {
+      flags |= kCmdBlocking;
+
+      // blocking commands should always be no-script
+      // TODO: we can relax this restriction if scripting becomes non-exclusive
+      flags |= kCmdNoScript;
+    } else {
       std::cout << fmt::format("Encountered non-existent flag '{}' in command {} in command attribute parsing", flag,
                                cmd_name)
                 << std::endl;
@@ -253,23 +350,21 @@ inline uint64_t ParseCommandFlags(const std::string &description, const std::str
   return flags;
 }
 
+template <typename T>
+auto MakeCmdAttr(const std::string &name, int arity, const std::string &description, NoKeyInThisCommand no_key,
+                 const AdditionalFlagGen &flag_gen = {}) {
+  CommandAttributes attr(name, arity, CommandCategory::Unknown, ParseCommandFlags(description, name), flag_gen, no_key,
+                         []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); });
+
+  return attr;
+}
+
 template <typename T>
 auto MakeCmdAttr(const std::string &name, int arity, const std::string &description, int first_key, int last_key,
                  int key_step = 1, const AdditionalFlagGen &flag_gen = {}) {
-  CommandAttributes attr{name,
-                         arity,
-                         CommandCategory::Unknown,
-                         ParseCommandFlags(description, name),
-                         flag_gen,
+  CommandAttributes attr(name, arity, CommandCategory::Unknown, ParseCommandFlags(description, name), flag_gen,
                          {first_key, last_key, key_step},
-                         {},
-                         {},
-                         []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); }};
-
-  if ((first_key > 0 && key_step <= 0) || (first_key > 0 && last_key >= 0 && last_key < first_key)) {
-    std::cout << fmt::format("Encountered invalid key range in command {}", name) << std::endl;
-    std::abort();
-  }
+                         []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); });
 
   return attr;
 }
@@ -282,9 +377,7 @@ auto MakeCmdAttr(const std::string &name, int arity, const std::string &descript
                          CommandCategory::Unknown,
                          ParseCommandFlags(description, name),
                          flag_gen,
-                         {-1, 0, 0},
                          gen,
-                         {},
                          []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); }};
 
   return attr;
@@ -298,8 +391,6 @@ auto MakeCmdAttr(const std::string &name, int arity, const std::string &descript
                          CommandCategory::Unknown,
                          ParseCommandFlags(description, name),
                          flag_gen,
-                         {-2, 0, 0},
-                         {},
                          vec_gen,
                          []() -> std::unique_ptr<Commander> { return std::unique_ptr<Commander>(new T()); }};
 
@@ -321,8 +412,8 @@ struct CommandTable {
   static void GetAllCommandsInfo(std::string *info);
   static void GetCommandsInfo(std::string *info, const std::vector<std::string> &cmd_names);
   static std::string GetCommandInfo(const CommandAttributes *command_attributes);
-  static Status GetKeysFromCommand(const CommandAttributes *attributes, const std::vector<std::string> &cmd_tokens,
-                                   std::vector<int> *keys_indexes);
+  static StatusOr<std::vector<int>> GetKeysFromCommand(const CommandAttributes *attributes,
+                                                       const std::vector<std::string> &cmd_tokens);
 
   static size_t Size();
   static bool IsExists(const std::string &name);
