@@ -25,12 +25,13 @@
 #include "db_util.h"
 
 namespace engine {
-DBIterator::DBIterator(Storage *storage, rocksdb::ReadOptions read_options, int slot)
-    : storage_(storage),
+DBIterator::DBIterator(engine::Context &ctx, rocksdb::ReadOptions read_options, int slot)
+    : storage_(ctx.storage),
       read_options_(std::move(read_options)),
+      ctx_(&ctx),
       slot_(slot),
       metadata_cf_handle_(storage_->GetCFHandle(ColumnFamilyID::Metadata)) {
-  metadata_iter_ = util::UniqueIterator(storage_->NewIterator(read_options_, metadata_cf_handle_));
+  metadata_iter_ = util::UniqueIterator(storage_->NewIterator(ctx, read_options_, metadata_cf_handle_));
 }
 
 void DBIterator::Next() {
@@ -110,18 +111,19 @@ std::unique_ptr<SubKeyIterator> DBIterator::GetSubKeyIterator() const {
     return nullptr;
   }
 
-  auto prefix = InternalKey(Key(), "", metadata_.version, storage_->IsSlotIdEncoded()).Encode();
-  return std::make_unique<SubKeyIterator>(storage_, read_options_, type, std::move(prefix));
+  auto prefix = InternalKey(Key(), "", metadata_.version, ctx_->storage->IsSlotIdEncoded()).Encode();
+  return std::make_unique<SubKeyIterator>(*ctx_, read_options_, type, std::move(prefix));
 }
 
-SubKeyIterator::SubKeyIterator(Storage *storage, rocksdb::ReadOptions read_options, RedisType type, std::string prefix)
-    : storage_(storage), read_options_(std::move(read_options)), type_(type), prefix_(std::move(prefix)) {
+SubKeyIterator::SubKeyIterator(engine::Context &ctx, rocksdb::ReadOptions read_options, RedisType type,
+                               std::string prefix)
+    : storage_(ctx.storage), read_options_(std::move(read_options)), type_(type), prefix_(std::move(prefix)) {
   if (type_ == kRedisStream) {
     cf_handle_ = storage_->GetCFHandle(ColumnFamilyID::Stream);
   } else {
     cf_handle_ = storage_->GetCFHandle(ColumnFamilyID::PrimarySubkey);
   }
-  iter_ = util::UniqueIterator(storage_->NewIterator(read_options_, cf_handle_));
+  iter_ = util::UniqueIterator(storage_->NewIterator(ctx, read_options_, cf_handle_));
 }
 
 void SubKeyIterator::Next() {
@@ -167,7 +169,8 @@ void SubKeyIterator::Reset() {
 }
 
 rocksdb::Status WALBatchExtractor::PutCF(uint32_t column_family_id, const Slice &key, const Slice &value) {
-  if (slot_ != -1 && slot_ != ExtractSlotId(key)) {
+  auto key_slot_id = ExtractSlotId(key);
+  if (slot_range_.IsValid() && !slot_range_.Contains(key_slot_id)) {
     return rocksdb::Status::OK();
   }
   items_.emplace_back(WALItem::Type::kTypePut, column_family_id, key.ToString(), value.ToString());
@@ -175,7 +178,8 @@ rocksdb::Status WALBatchExtractor::PutCF(uint32_t column_family_id, const Slice
 }
 
 rocksdb::Status WALBatchExtractor::DeleteCF(uint32_t column_family_id, const rocksdb::Slice &key) {
-  if (slot_ != -1 && slot_ != ExtractSlotId(key)) {
+  auto key_slot_id = ExtractSlotId(key);
+  if (slot_range_.IsValid() && !slot_range_.Contains(key_slot_id)) {
     return rocksdb::Status::OK();
   }
   items_.emplace_back(WALItem::Type::kTypeDelete, column_family_id, key.ToString(), std::string{});
@@ -245,7 +249,7 @@ void WALIterator::nextBatch() {
 }
 
 void WALIterator::Seek(rocksdb::SequenceNumber seq) {
-  if (slot_ != -1 && !storage_->IsSlotIdEncoded()) {
+  if (slot_range_.IsValid() && !storage_->IsSlotIdEncoded()) {
     Reset();
     return;
   }
