@@ -43,8 +43,6 @@ func SetBinaryBuffer(buf *bytes.Buffer, vec []float64) error {
 }
 
 func TestSearch(t *testing.T) {
-	t.Skip("search commands is disabled")
-
 	srv := util.StartServer(t, map[string]string{})
 	defer srv.Close()
 
@@ -63,14 +61,12 @@ func TestSearch(t *testing.T) {
 			idxInfo := infoRes.Val().([]interface{})
 			require.Equal(t, "index_name", idxInfo[0])
 			require.Equal(t, "testidx1", idxInfo[1])
-			require.Equal(t, "on_data_type", idxInfo[2])
-			require.Equal(t, "ReJSON-RL", idxInfo[3])
-			require.Equal(t, "prefixes", idxInfo[4])
-			require.Equal(t, []interface{}{"test1:"}, idxInfo[5])
-			require.Equal(t, "fields", idxInfo[6])
-			require.Equal(t, []interface{}{"a", "tag"}, idxInfo[7].([]interface{})[0])
-			require.Equal(t, []interface{}{"b", "numeric"}, idxInfo[7].([]interface{})[1])
-			require.Equal(t, []interface{}{"c", "vector"}, idxInfo[7].([]interface{})[2])
+			require.Equal(t, "index_definition", idxInfo[2])
+			require.Equal(t, []interface{}{"key_type", "ReJSON-RL", "prefixes", []interface{}{"test1:"}}, idxInfo[3])
+			require.Equal(t, "fields", idxInfo[4])
+			require.Equal(t, []interface{}{"identifier", "a", "type", "tag", "properties", []interface{}{"separator", ",", "case_sensitive", int64(0)}}, idxInfo[5].([]interface{})[0])
+			require.Equal(t, []interface{}{"identifier", "b", "type", "numeric", "properties", []interface{}{}}, idxInfo[5].([]interface{})[1])
+			require.Equal(t, []interface{}{"identifier", "c", "type", "vector", "properties", []interface{}{"algorithm", "HNSW", "vector_type", "FLOAT64", "dim", int64(3), "distance_metric", "L2", "m", int64(16), "ef_construction", int64(200), "ef_runtime", int64(10), "epsilon", "0.01"}}, idxInfo[5].([]interface{})[2])
 		}
 		verify(t)
 
@@ -158,6 +154,44 @@ func TestSearch(t *testing.T) {
 		verify(t, res)
 	})
 
+	t.Run("FT.TAGVALS with updated index", func(t *testing.T) {
+		require.NoError(t, rdb.Do(ctx, "JSON.SET", "test1:k1", "$", `{"a": "x,y", "b": 11, "c": [1, 2, 3]}`).Err())
+		require.NoError(t, rdb.Do(ctx, "JSON.SET", "test1:k2", "$", `{"a": "x,z", "b": 22, "c": [4, 5, 6]}`).Err())
+		require.NoError(t, rdb.Do(ctx, "JSON.SET", "test1:k3", "$", `{"a": "y,z", "b": 33, "c": [7, 8, 9]}`).Err())
+		require.NoError(t, rdb.Do(ctx, "JSON.SET", "test1:k4", "$", `{"a": "a,b,c", "b": 44, "c": [10, 11, 12]}`).Err())
+
+		// Helper function to verify tag values
+		verifyTagVals := func(t *testing.T, res *redis.Cmd, expected map[string]struct{}) {
+			require.NoError(t, res.Err())
+			values, err := res.Result()
+			require.NoError(t, err)
+			for _, val := range values.([]interface{}) {
+				tag := val.(string)
+				_, exists := expected[tag]
+				require.True(t, exists, "Unexpected tag value: %s", tag)
+				delete(expected, tag)
+			}
+			require.Empty(t, expected, "Missing expected tag values")
+		}
+
+		// Query tag values for the field 'a', which is indexed as a tag field
+		res := rdb.Do(ctx, "FT.TAGVALS", "testidx1", "a")
+		expectedA := map[string]struct{}{
+			"a": {}, "b": {}, "c": {}, "x": {}, "y": {}, "z": {},
+		}
+		verifyTagVals(t, res, expectedA)
+
+		// Querying tag values for the field 'b', which is a numeric field, should return an empty result.
+		res = rdb.Do(ctx, "FT.TAGVALS", "testidx1", "b")
+		expectedB := map[string]struct{}{}
+		verifyTagVals(t, res, expectedB)
+
+		// Querying tag values for the field 'c', which is a vector field, should return an empty result.
+		res = rdb.Do(ctx, "FT.TAGVALS", "testidx1", "c")
+		expectedC := map[string]struct{}{}
+		verifyTagVals(t, res, expectedC)
+	})
+
 	t.Run("FT.DROPINDEX", func(t *testing.T) {
 		require.NoError(t, rdb.Do(ctx, "FT.DROPINDEX", "testidx1").Err())
 
