@@ -31,7 +31,6 @@ import (
 	"github.com/apache/kvrocks/tests/gocase/util"
 	"github.com/redis/go-redis/v9"
 	"github.com/stretchr/testify/require"
-	"modernc.org/mathutil"
 )
 
 // We need a value larger than list-max-ziplist-value to make sure
@@ -85,7 +84,7 @@ func testLTRIM(t *testing.T, configs util.KvrocksServerConfigs) {
 				lo := int64(rand.Float64() * float64(startLen))
 				hi := int64(float64(lo) + rand.Float64()*float64(startLen))
 
-				myList = myList[lo:mathutil.Min(int(hi+1), len(myList))]
+				myList = myList[lo:min(int(hi+1), len(myList))]
 				require.NoError(t, rdb.LTrim(ctx, key, lo, hi).Err())
 				require.Equal(t, myList, rdb.LRange(ctx, key, 0, -1).Val(), "failed trim")
 
@@ -376,22 +375,13 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			createList("blist", []string{"a", "b", large, "c", "d"})
-			// TODO: Remove time.Sleep after fix issue #2473
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blpop", "blist", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist", "a"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("brpop", "blist", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist", "d"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blpop", "blist", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist", "b"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("brpop", "blist", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist", "c"})
 		})
 
@@ -400,23 +390,15 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			defer func() { require.NoError(t, rd.Close()) }()
 			createList("blist1", []string{"a", large, "c"})
 			createList("blist2", []string{"d", large, "f"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blpop", "blist1", "blist2", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist1", "a"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("brpop", "blist1", "blist2", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist1", "c"})
 			require.EqualValues(t, 1, rdb.LLen(ctx, "blist1").Val())
 			require.EqualValues(t, 3, rdb.LLen(ctx, "blist2").Val())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blpop", "blist2", "blist2", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist2", "d"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("brpop", "blist2", "blist2", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist2", "f"})
 			require.EqualValues(t, 1, rdb.LLen(ctx, "blist1").Val())
 			require.EqualValues(t, 1, rdb.LLen(ctx, "blist2").Val())
@@ -427,13 +409,9 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, "blist1").Err())
 			createList("blist2", []string{"d", large, "f"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blpop", "blist1", "blist2", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist2", "d"})
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("brpop", "blist1", "blist2", "1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustReadStrings(t, []string{"blist2", "f"})
 			require.EqualValues(t, 0, rdb.LLen(ctx, "blist1").Val())
 			require.EqualValues(t, 1, rdb.LLen(ctx, "blist2").Val())
@@ -444,25 +422,17 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 		rd := srv.NewTCPClient()
 		defer func() { require.NoError(t, rd.Close()) }()
 		require.NoError(t, rdb.Del(ctx, "list1", "list2").Err())
-		time.Sleep(time.Millisecond * 100)
 		require.NoError(t, rd.WriteArgs("blpop", "list1", "list2", "list2", "list1", "0"))
-		time.Sleep(time.Millisecond * 100)
 		require.NoError(t, rdb.LPush(ctx, "list1", "a").Err())
 		rd.MustReadStrings(t, []string{"list1", "a"})
-		time.Sleep(time.Millisecond * 100)
 		require.NoError(t, rd.WriteArgs("blpop", "list1", "list2", "list2", "list1", "0"))
-		time.Sleep(time.Millisecond * 100)
 		require.NoError(t, rdb.LPush(ctx, "list2", "b").Err())
 		rd.MustReadStrings(t, []string{"list2", "b"})
 		require.NoError(t, rdb.LPush(ctx, "list1", "a").Err())
 		require.NoError(t, rdb.LPush(ctx, "list2", "b").Err())
-		time.Sleep(time.Millisecond * 100)
 		require.NoError(t, rd.WriteArgs("blpop", "list1", "list2", "list2", "list1", "0"))
-		time.Sleep(time.Millisecond * 100)
 		rd.MustReadStrings(t, []string{"list1", "a"})
-		time.Sleep(time.Millisecond * 100)
 		require.NoError(t, rd.WriteArgs("blpop", "list1", "list2", "list2", "list1", "0"))
-		time.Sleep(time.Millisecond * 100)
 		rd.MustReadStrings(t, []string{"list2", "b"})
 	})
 
@@ -470,9 +440,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 		rd := srv.NewTCPClient()
 		defer func() { require.NoError(t, rd.Close()) }()
 		require.NoError(t, rdb.Del(ctx, "blist", "target").Err())
-		time.Sleep(time.Millisecond * 100)
 		require.NoError(t, rd.WriteArgs("blpop", "blist", "0"))
-		time.Sleep(time.Millisecond * 100)
 		require.EqualValues(t, 2, rdb.LPush(ctx, "blist", "foo", "bar").Val())
 		rd.MustReadStrings(t, []string{"blist", "bar"})
 		require.Equal(t, "foo", rdb.LRange(ctx, "blist", 0, -1).Val()[0])
@@ -483,9 +451,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, "blist1").Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs(popType, "blist1", "0"))
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rdb.RPush(ctx, "blist1", "foo").Err())
 			rd.MustReadStrings(t, []string{"blist1", "foo"})
 			require.EqualValues(t, 0, rdb.Exists(ctx, "blist1").Val())
@@ -495,7 +461,6 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rd.WriteArgs(popType, "blist1", "-1"))
-			time.Sleep(100 * time.Millisecond)
 			rd.MustMatch(t, ".*negative.*")
 		})
 
@@ -505,7 +470,6 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rd.WriteArgs(popType, "blist1", "0"))
-			time.Sleep(time.Millisecond * 1000)
 			require.NoError(t, rdb.RPush(ctx, "blist1", "foo").Err())
 			rd.MustReadStrings(t, []string{"blist1", "foo"})
 		})
@@ -515,7 +479,6 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, "blist1", "blist2").Err())
 			require.NoError(t, rdb.Set(ctx, "blist2", "nolist", 0).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs(popType, "blist1", "blist2", "1"))
 			rd.MustMatch(t, ".*WRONGTYPE.*")
 		})
@@ -524,7 +487,6 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, "blist1", "blist2").Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs(popType, "blist1", "blist2", "1"))
 			rd.MustMatch(t, "")
 		})
@@ -533,16 +495,12 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, "blist1", "blist2").Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs(popType, "blist1", "blist2", "4"))
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rdb.RPush(ctx, "blist1", "foo").Err())
 			rd.MustReadStrings(t, []string{"blist1", "foo"})
 			require.EqualValues(t, 0, rdb.Exists(ctx, "blist1").Val())
 			require.EqualValues(t, 0, rdb.Exists(ctx, "blist2").Val())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs(popType, "blist1", "blist2", "1"))
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rdb.RPush(ctx, "blist2", "foo").Err())
 			rd.MustReadStrings(t, []string{"blist2", "foo"})
 			require.EqualValues(t, 0, rdb.Exists(ctx, "blist1").Val())
@@ -818,10 +776,10 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 	})
 
 	for listType, large := range largeValue {
-		trimList := func(listType string, min, max int64) []string {
+		trimList := func(listType string, minInt, maxInt int64) []string {
 			require.NoError(t, rdb.Del(ctx, "mylist").Err())
 			createList("mylist", []string{"1", "2", "3", "4", large})
-			require.NoError(t, rdb.LTrim(ctx, "mylist", min, max).Err())
+			require.NoError(t, rdb.LTrim(ctx, "mylist", minInt, maxInt).Err())
 			return rdb.LRange(ctx, "mylist", 0, -1).Val()
 		}
 
@@ -950,9 +908,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 				require.NoError(t, rdb.Del(ctx, "target_key{t}").Err())
 				require.NoError(t, rdb.RPush(ctx, "target_key{t}", 1).Err())
 				createList("list{t}", []string{"a", "b", "c", "d"})
-				time.Sleep(100 * time.Millisecond)
 				require.NoError(t, rd.WriteArgs("lmove", "list{t}", "target_key{t}", from, to))
-				time.Sleep(100 * time.Millisecond)
 				r, err1 := rd.ReadLine()
 				require.Equal(t, "$1", r)
 				require.NoError(t, err1)
@@ -998,9 +954,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 				require.NoError(t, rdb.Del(ctx, "target_key{t}").Err())
 				require.NoError(t, rdb.RPush(ctx, "target_key{t}", 1).Err())
 				createList("list{t}", []string{"a", "b", "c", "d"})
-				time.Sleep(100 * time.Millisecond)
 				require.NoError(t, rd.WriteArgs("blmove", "list{t}", "target_key{t}", from, to, "1"))
-				time.Sleep(100 * time.Millisecond)
 				r, err1 := rd.ReadLine()
 				require.Equal(t, "$1", r)
 				require.NoError(t, err1)
@@ -1026,9 +980,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 		rd := srv.NewTCPClient()
 		defer func() { require.NoError(t, rd.Close()) }()
 		require.NoError(t, rdb.Del(ctx, "blist", "target").Err())
-		time.Sleep(100 * time.Millisecond)
 		require.NoError(t, rd.WriteArgs("blmove", "blist", "target", "left", "right", "0"))
-		time.Sleep(100 * time.Millisecond)
 		require.EqualValues(t, 2, rdb.LPush(ctx, "blist", "foo", "bar").Val())
 		rd.MustRead(t, "$3")
 		require.Equal(t, "bar", rdb.LRange(ctx, "target", 0, -1).Val()[0])
@@ -1436,9 +1388,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "1", key1, direction, "count", "1"))
-			time.Sleep(time.Millisecond * 100)
 			require.NoError(t, rdb.RPush(ctx, key1, "ONE", "TWO").Err())
 			if direction == "LEFT" {
 				rd.MustReadStringsWithKey(t, key1, []string{"ONE"})
@@ -1452,9 +1402,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "1", key1, direction, "count", "2"))
-			time.Sleep(time.Millisecond * 100)
 			require.NoError(t, rdb.RPush(ctx, key1, "ONE", "TWO").Err())
 			if direction == "LEFT" {
 				rd.MustReadStringsWithKey(t, key1, []string{"ONE", "TWO"})
@@ -1468,9 +1416,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "1", key1, direction, "count", "10"))
-			time.Sleep(time.Millisecond * 100)
 			require.NoError(t, rdb.RPush(ctx, key1, "ONE", "TWO").Err())
 			if direction == "LEFT" {
 				rd.MustReadStringsWithKey(t, key1, []string{"ONE", "TWO"})
@@ -1484,9 +1430,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "2", key1, key2, direction, "count", "2"))
-			time.Sleep(time.Millisecond * 100)
 			require.NoError(t, rdb.RPush(ctx, key1, "ONE", "TWO").Err())
 			if direction == "LEFT" {
 				rd.MustReadStringsWithKey(t, key1, []string{"ONE", "TWO"})
@@ -1500,9 +1444,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "2", key1, key2, direction, "count", "2"))
-			time.Sleep(time.Millisecond * 100)
 			require.NoError(t, rdb.RPush(ctx, key2, "one", "two").Err())
 			if direction == "LEFT" {
 				rd.MustReadStringsWithKey(t, key2, []string{"one", "two"})
@@ -1516,9 +1458,10 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "2", key1, key2, direction, "count", "2"))
-			time.Sleep(time.Millisecond * 100)
+			// https://github.com/apache/kvrocks/issues/2617
+			// WriteArgs are required to be executed first
+			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rdb.RPush(ctx, key2, "one", "two").Err())
 			require.NoError(t, rdb.RPush(ctx, key1, "ONE", "TWO").Err())
 			if direction == "LEFT" {
@@ -1534,9 +1477,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "2", key1, key2, direction))
-			time.Sleep(time.Millisecond * 100)
 			require.NoError(t, rdb.RPush(ctx, key2, "one", "two").Err())
 			if direction == "LEFT" {
 				rd.MustReadStringsWithKey(t, key2, []string{"one"})
@@ -1551,9 +1492,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "1", "2", key1, key2, direction))
-			time.Sleep(time.Millisecond * 1200)
 			rd.MustMatch(t, "")
 		})
 
@@ -1562,9 +1501,7 @@ func testList(t *testing.T, configs util.KvrocksServerConfigs) {
 			rd := srv.NewTCPClient()
 			defer func() { require.NoError(t, rd.Close()) }()
 			require.NoError(t, rdb.Del(ctx, key1, key2).Err())
-			time.Sleep(100 * time.Millisecond)
 			require.NoError(t, rd.WriteArgs("blmpop", "0", "2", key1, key2, direction, "count", "2"))
-			time.Sleep(time.Millisecond * 1200)
 			require.NoError(t, rdb.RPush(ctx, key2, "one", "two").Err())
 			if direction == "LEFT" {
 				rd.MustReadStringsWithKey(t, key2, []string{"one", "two"})
